require "sinbotra/server"
require "sucker_punch"

logger = Logger.new(STDERR)
logger.level = :debug
SuckerPunch.logger = logger

class PushupBot
  include Sinbotra::Messenger::Handler

  enable_get_started!

  get_started do
    start_conversation(:onboarding)
  end

  hear(/hi|hello|yo/i) do |bot|
    bot.say "Hello there!"
    bot.start_conversation(:onboarding)
  end

  hear(/start/i) do |bot|
    bot.start_conversation(:start_day)
  end

  hear do |bot|
    msg = bot.current_message
    intent = get_intent(msg.text)
    if bot.has_conversation?(intent)
      bot.start_conversation(intent)
    else
      bot.say next_dont_understand_phrase
    end
  end

  conversation :onboarding do |convo|
    convo.step :ask_name do |_bot|
      convo.say "Hi there!"
      convo.say "What's your name?"
      convo.next_step!
    end

    convo.step :ask_height do |bot|
      convo.store!(:name, bot.current_message.text)
      convo.say "Great #{convo.recall(:name)}. How tall are you?"
      convo.next_step!
    end

    convo.step :done do |bot|
      convo.store!(:height, bot.current_message.text)
      convo.say "Thank you for the info"
      typing 1
      convo.say "See you around!"
      convo.done!
    end
  end

  conversation :start_day do |convo|
    convo.step :how_many do |_bot|
      convo.say "How many pomodoros would you like to do today?"
      convo.next_step!
    end

    convo.step :ask_start_now do |bot|
      count = bot.current_message.text.to_i
      if count < 1
        convo.say("I totally didn't understand that. How many would you like to do?")
        convo.repeat_step!
      else
        convo.store!(:pomodoro_count, count)
        convo.say "Would you like to start now?"
        convo.next_step!
      end
    end

    convo.step :now_yes_no do |bot|
      if yes?(bot.current_message.text)
        count = convo.recall(:pomodoro_count)
        convo.say "Great #{convo.recall(:name)}. I'll set you up for #{count} pomodoros."
        convo.say "I'll get back to you in 25 minutes. Cheers"
      else
        bot.quick_replies("Ok, let me know when you'd like to start.", ["Start Pomodoros", "Start Over"])
      end
      convo.done!
    end
  end
end

Sinbotra::Bot.add_handler(:facebook, PushupBot)

run Sinbotra::Server
