class Sinbotra::Bot
  class Conversation
    class << self
      attr_reader :_steps

      private

      def steps(*_steps)
        @_steps = _steps
      end
    end

    def initialize(bot, platform)
      @bot          = bot
      @current_user = bot.current_user
      @message      = bot.message
      @platform     = platform
    end

    def start
      run_step!(current_user.conversation.step, message)
    end

    def continue_dialogue
      run_step!(current_user.conversation.step, message)
    end

    private

    attr_reader :current_user
    attr_reader :message
    attr_reader :platform

    def start_conversation(convo_id)
      @bot.start_conversation(convo_id)
    end

    def has_conversation?(intent)
      @bot.has_conversation(intent)
    end

    def get_intent(msg)
      @bot.get_intent(msg)
    end

    def steps; self.class._steps; end

    def step(n) steps[n]; end

    def step_count; steps.size; end

    def run_step!(step_number, message)
      send(step(step_number), message)
    end

    def skip_to_next_step!
      next_step!
    end

    def has_steps?
      current_user.conversation.step < step_count - 1
    end

    def next_step!
      has_steps? ? current_user.next_step! : done!
    end

    def next_step_now!
      if has_steps?
        current_user.next_step!
        run_step!(current_user.conversation.step, message)
      else
        raise ArgumentError, "We're out of steps, shouldn't be skipping: #{self}"
      end
    end

    def repeat_step!
      # do nothing, leave state the same
    end

    def done!
      current_user.end_conversation!
    end
  end
end
