require "openssl"

module Sinbotra::Messenger::Middleware
  class FacebookSignature
    def initialize(app)
      raise ArgumentError, "You need to set a FACEBOOK_PAGE_TOKEN environmental variable to run the server!" unless ENV["FACEBOOK_PAGE_TOKEN"]
      raise ArgumentError, "You need to set a FACEBOOK_APP_SECRET environmental variable to run the server!" unless ENV["FACEBOOK_APP_SECRET"]
      @app = app
    end

    def call(env)
      req = Rack::Request.new(env)
      return @app.call(env) unless req.post?

      payload = req.body.read
      unless signature_valid?(payload, env)
        return Rack::Response.new([], 401, {}).finish
      end
      @app.call(env)
    end

    def signature_valid?(payload_body, env)
      digest = OpenSSL::HMAC.hexdigest(
        OpenSSL::Digest.new("sha1"),
        ENV["FACEBOOK_APP_SECRET"],
        payload_body
      )
      signature = "sha1=" + digest
      Rack::Utils.secure_compare(signature, env["HTTP_X_HUB_SIGNATURE"])
    end
  end
end
