module Sinbotra::Messenger
  class Platform
    attr_reader :recipient_id
    attr_reader :client

    def self.get_started(postback_id)
      client = MessengerClient::Client.new(ENV.fetch("FACEBOOK_PAGE_TOKEN"))
      client.get_started(postback_id)
    end

    def initialize(recipient_id)
      @recipient_id = recipient_id
      @client = MessengerClient::Client.new(ENV.fetch("FACEBOOK_PAGE_TOKEN"))
    end

    def say(text)
      client.text(
        recipient_id: recipient_id,
        text: text
      )
    end
    alias_method :text, :say

    def quick_replies(text, replies)
      client.qr(
        recipient_id: recipient_id,
        text:         text,
        replies:      replies
      )
    end

    def typing(seconds=0, &blk)
      client.typing(recipient_id: recipient_id, seconds: seconds, &blk)
    end

    def qr(text, postback=nil)
      MessengerClient::QuickReply.new(text, postback)
    end

    def payload_button(text, payload)
      MessengerClient::PayloadButton.new(text, payload)
    end

    def url_button(text, url)
      MessengerClient::URLButton.new(text, url)
    end

    def generic_template(title, subtitle, image, link, buttons)
    end

    def text_with_buttons(text, buttons)
      client.button_template(
        recipient_id: recipient_id,
        text:         text,
        buttons:      buttons
      )
    end

    def video(url)
      client.video(recipient_id: recipient_id, url: url)
    end

    def image(url)
      client.image(recipient_id: recipient_id, url: url)
    end

    def location
      client.location(recipient_id: recipient_id)
    end
  end
end

